
#ifndef ANGLEPOTENTIAL_H
#define ANGLEPOTENTIAL_H

#include "rumd/Potential.h"

class MoleculeData; 

///////////////////////////////////////////////////////
// Abstract BondPotential class
///////////////////////////////////////////////////////

class AnglePotential : public Potential{
 public:
  MoleculeData *M;
  unsigned num_angles;
  unsigned num_blocks, num_threads;
  bool EnergyIncludedInParticleSum() const { return true; };
};


// Cosine squared potential
class AngleCosSq : public AnglePotential{
 private:
  AngleCosSq(const AngleCosSq&);
  AngleCosSq& operator=(const AngleCosSq&);
  
 public:
  AngleCosSq(MoleculeData *);
  float CalcF(bool initialize, bool calc_stresses, bool report_time);
  double GetPotentialEnergy();
};


///////////////////////////////////////////////////////
// Kernel prototypes
///////////////////////////////////////////////////////

template <int energy, class Simbox> 
__global__ void CosSq( float4 *r, float4 *f,
		       uint4 *alist, float2 *parameter,
		       float *d_epot_angle, float *d_angles,
		       unsigned num_angles, Simbox *simbox, float *simboxpointer );


//////////////////////////////////////////////////////////
//
///////////////////////////////////////////////////////
__global__ void AngleResetForces( float4 *f, float4 *w );
__device__ inline void AngleAtomicFloatAdd(float *address, float val);

#endif
